"""Define system models."""
# pylint: disable=consider-alternative-union-syntax
from __future__ import annotations

from datetime import datetime
from typing import Optional

from pydantic import Field, field_validator

from aionotion.helpers.model import NotionBaseModel
from aionotion.helpers.validator import validate_timestamp


class System(NotionBaseModel):
    """Define a system."""

    uuid: str
    name: str
    mode: str
    partners: list[str]
    latitude: float
    longitude: float
    timezone_id: str
    created_at: datetime
    updated_at: datetime
    night_time_start: datetime
    night_time_end: datetime
    id: int
    locality: str
    postal_code: str
    administrative_area: str
    fire_number: str
    police_number: str
    emergency_number: str
    address: Optional[str]
    notion_pro_permit: Optional[str]

    validate_created_at = field_validator("created_at", mode="before")(
        validate_timestamp
    )
    validate_updated_at = field_validator("updated_at", mode="before")(
        validate_timestamp
    )
    validate_night_time_start = field_validator("night_time_start", mode="before")(
        validate_timestamp
    )
    validate_night_time_end = field_validator("night_time_end", mode="before")(
        validate_timestamp
    )


class SystemAllResponse(NotionBaseModel):
    """Define an API response containing all systems."""

    systems: list[System]


class SystemGetResponse(NotionBaseModel):
    """Define an API response containing a single system."""

    system: System = Field(alias="systems")



"""Define user models."""
from __future__ import annotations

from aionotion.helpers.model import NotionBaseModel


class UserPreferences(NotionBaseModel):
    """Define user preferences."""

    user_id: int
    military_time_enabled: bool
    celsius_enabled: bool
    disconnect_alerts_enabled: bool
    home_away_alerts_enabled: bool
    battery_alerts_enabled: bool


class UserPreferencesResponse(NotionBaseModel):
    """Define an API response containing all devices."""

    user_preferences: UserPreferences



"""Define sensor models."""
# pylint: disable=consider-alternative-union-syntax
from __future__ import annotations

from datetime import datetime
from enum import Enum
from typing import Any, Literal, Optional

from pydantic import ConfigDict, Field, field_validator

from aionotion.const import LOGGER
from aionotion.helpers.model import NotionBaseModel
from aionotion.helpers.validator import validate_timestamp


class Bridge(NotionBaseModel):
    """Define a bridge representation."""

    id: int
    hardware_id: str


class Firmware(NotionBaseModel):
    """Define firmware information."""

    status: str


class SurfaceType(NotionBaseModel):
    """Define a surface type."""

    id: str
    name: str
    slug: str


class User(NotionBaseModel):
    """Define a user representation."""

    id: int
    email: str


class Sensor(NotionBaseModel):
    """Define a sensor."""

    id: int
    uuid: str
    user: User
    bridge: Bridge
    last_bridge_hardware_id: str
    name: str
    location_id: int
    system_id: int
    hardware_id: str
    hardware_revision: int
    firmware_version: str
    device_key: str
    encryption_key: bool
    installed_at: Optional[datetime]
    calibrated_at: Optional[datetime]
    last_reported_at: Optional[datetime]
    missing_at: Optional[datetime]
    updated_at: datetime
    created_at: datetime
    signal_strength: int
    firmware: Firmware
    surface_type: Optional[SurfaceType]

    validate_installed_at = field_validator("installed_at", mode="before")(
        validate_timestamp
    )
    validate_calibrated_at = field_validator("calibrated_at", mode="before")(
        validate_timestamp
    )
    validate_last_reported_at = field_validator("last_reported_at", mode="before")(
        validate_timestamp
    )
    validate_missing_at = field_validator("missing_at", mode="before")(
        validate_timestamp
    )
    validate_created_at = field_validator("created_at", mode="before")(
        validate_timestamp
    )
    validate_updated_at = field_validator("updated_at", mode="before")(
        validate_timestamp
    )


class SensorAllResponse(NotionBaseModel):
    """Define an API response containing all sensors."""

    sensors: list[Sensor] = Field(alias="sensors")


class SensorGetResponse(NotionBaseModel):
    """Define an API response containing a single sensor."""

    sensor: Sensor = Field(alias="sensors")


class ListenerStatus(NotionBaseModel, extra="allow"):
    """Define a listener status."""

    trigger_value: str
    data_received_at: datetime

    validate_data_received_at = field_validator("data_received_at", mode="before")(
        validate_timestamp
    )


class ListenerLocalizedStatus(NotionBaseModel):
    """Define a localized listener status."""

    state: str
    description: str


class InsightOrigin(NotionBaseModel):
    """Define an insight origin."""

    type: Optional[str]
    id: Optional[str]


class PrimaryListenerInsight(NotionBaseModel):
    """Define a primary listener insight."""

    origin: Optional[InsightOrigin]
    value: Optional[str]
    data_received_at: Optional[datetime]

    validate_data_received_at = field_validator("data_received_at", mode="before")(
        validate_timestamp
    )


class ListenerInsights(NotionBaseModel):
    """Define listener insights:"""

    primary: PrimaryListenerInsight


class ListenerKind(Enum):
    """Define the kinds of listener."""

    BATTERY = 0
    MOLD = 2
    TEMPERATURE = 3
    LEAK_STATUS = 4
    SAFE = 5
    DOOR = 6
    SMOKE = 7
    CONNECTED = 10
    HINGED_WINDOW = 12
    GARAGE_DOOR = 13
    SLIDING_DOOR_OR_WINDOW = 32
    UNKNOWN = 99


class Listener(NotionBaseModel):
    """Define a listener."""

    model_config = ConfigDict(frozen=True, protected_namespaces=())

    id: str
    created_at: datetime
    model_version: str
    sensor_id: str
    insights: ListenerInsights
    configuration: dict[str, Any]
    pro_monitoring_status: Literal["eligible", "ineligible"]

    device_type: str = Field(alias="type")
    listener_kind: ListenerKind = Field(alias="definition_id")
    status: Optional[ListenerStatus] = None
    status_localized: Optional[ListenerLocalizedStatus] = None

    @field_validator("listener_kind", mode="before")
    @classmethod
    def validate_listener_kind(cls, value: str) -> ListenerKind:
        """Validate the API key type.

        Args:
            value: An API key to validate.

        Returns:
            A parsed ApiKeyType.

        Raises:
            ValueError: An invalid API key type was received.
        """
        try:
            return ListenerKind(value)
        except ValueError:
            LOGGER.warning("Received an unknown listener kind: %s", value)
            return ListenerKind.UNKNOWN

    validate_created_at = field_validator("created_at", mode="before")(
        validate_timestamp
    )


class ListenerAllResponse(NotionBaseModel):
    """Define an API response containing all listeners."""

    listeners: list[Listener]



"""Define device models."""
from __future__ import annotations

from datetime import datetime

from pydantic import Field, field_validator

from aionotion.helpers.model import NotionBaseModel
from aionotion.helpers.validator import validate_timestamp


class Device(NotionBaseModel):
    """Define a device."""

    id: int
    token: str
    platform: str
    endpoint: str
    created_at: datetime
    updated_at: datetime

    validate_created_at = field_validator("created_at", mode="before")(
        validate_timestamp
    )
    validate_updated_at = field_validator("updated_at", mode="before")(
        validate_timestamp
    )


class DeviceAllResponse(NotionBaseModel):
    """Define an API response containing all devices."""

    devices: list[Device]


class DeviceGetResponse(NotionBaseModel):
    """Define an API response containing a single device."""

    device: Device = Field(alias="devices")



"""Define endpoints for interacting with bridges."""
from __future__ import annotations

from typing import TYPE_CHECKING, Any

from aionotion.bridge.models import BridgeAllResponse, BridgeGetResponse

if TYPE_CHECKING:
    from aionotion.client import Client


class Bridge:
    """Define an object to interact with bridge endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> BridgeAllResponse:
        """Get all bridges.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", "base_stations", BridgeAllResponse
        )

    async def async_create(self, attributes: dict[str, Any]) -> BridgeGetResponse:
        """Create a bridge with a specific attribute payload.

        Args:
            attributes: The attributes to assign to the new bridge.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "post",
            "base_stations",
            BridgeGetResponse,
            json={"base_stations": attributes},
        )

    async def async_delete(self, bridge_id: int) -> None:
        """Delete a bridge by ID.

        Args:
            bridge_id: The ID of the bridge to delete.
        """
        await self._client.async_request("delete", f"base_stations/{bridge_id}")

    async def async_get(self, bridge_id: int) -> BridgeGetResponse:
        """Get a bridge by ID.

        Args:
            bridge_id: The ID of the bridge to get.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", f"base_stations/{bridge_id}", BridgeGetResponse
        )

    async def async_reset(self, bridge_id: int) -> BridgeGetResponse:
        """Reset a bridge (clear its wifi credentials) by ID.

        Args:
            bridge_id: The ID of the bridge to reset.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "put", f"base_stations/{bridge_id}/reset", BridgeGetResponse
        )

    async def async_update(
        self, bridge_id: int, new_attributes: dict[str, Any]
    ) -> BridgeGetResponse:
        """Update a bridge with a specific attribute payload.

        Args:
            bridge_id: The ID of the bridge to update.
            new_attributes: The new attributes to give the bridge.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "put",
            f"base_stations/{bridge_id}",
            BridgeGetResponse,
            json={"base_stations": new_attributes},
        )



"""Define bridge models."""
# pylint: disable=consider-alternative-union-syntax
from __future__ import annotations

from datetime import datetime
from typing import Optional, Union

from pydantic import Field, field_validator

from aionotion.helpers.model import NotionBaseModel
from aionotion.helpers.validator import validate_timestamp


class FirmwareVersion(NotionBaseModel):
    """Define firmware version info."""

    wifi: str
    wifi_app: str
    silabs: Optional[str] = None
    ti: Optional[str] = None


class Bridge(NotionBaseModel):
    """Define a bridge."""

    id: int
    name: Optional[str]
    mode: str
    hardware_id: str
    hardware_revision: int
    firmware_version: FirmwareVersion
    missing_at: Optional[datetime]
    created_at: datetime
    updated_at: datetime
    system_id: int
    firmware: FirmwareVersion
    links: dict[str, Union[int, str]]

    validate_missing_at = field_validator("missing_at", mode="before")(
        validate_timestamp
    )
    validate_created_at = field_validator("created_at", mode="before")(
        validate_timestamp
    )
    validate_updated_at = field_validator("updated_at", mode="before")(
        validate_timestamp
    )


class BridgeAllResponse(NotionBaseModel):
    """Define an API response containing all bridges."""

    bridges: list[Bridge] = Field(alias="base_stations")


class BridgeGetResponse(NotionBaseModel):
    """Define an API response containing a single bridge."""

    bridge: Bridge = Field(alias="base_stations")



"""Define endpoints for interacting with devices (phones, etc. with Notion)."""
from __future__ import annotations

from typing import TYPE_CHECKING, Any

from aionotion.device.models import DeviceAllResponse, DeviceGetResponse

if TYPE_CHECKING:
    from aionotion.client import Client


class Device:
    """Define an object to interact with device endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> DeviceAllResponse:
        """Get all devices.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", "devices", DeviceAllResponse
        )

    async def async_create(self, attributes: dict[str, Any]) -> DeviceGetResponse:
        """Create a device with a specific attribute payload.

        Args:
            attributes: The attributes to assign to the new device.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "post", "devices", DeviceGetResponse, json={"devices": attributes}
        )

    async def async_delete(self, device_id: int) -> None:
        """Delete a device by ID.

        Args:
            device_id: The ID of the device to delete.
        """
        await self._client.async_request("delete", f"devices/{device_id}")

    async def async_get(self, device_id: int) -> DeviceGetResponse:
        """Get a device by ID.

        Args:
            device_id: The ID of the device to get.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", f"devices/{device_id}", DeviceGetResponse
        )



"""Define endpoints for interacting with systems (accounts)."""
from __future__ import annotations

from typing import TYPE_CHECKING, Any

from aionotion.system.models import SystemAllResponse, SystemGetResponse

if TYPE_CHECKING:
    from aionotion.client import Client


class System:
    """Define an object to interact with system endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> SystemAllResponse:
        """Get all systems.

        Returns:
            An API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", "systems", SystemAllResponse
        )

    async def async_create(self, attributes: dict[str, Any]) -> SystemGetResponse:
        """Create a system with a specific attribute payload.

        Args:
            attributes: The attributes to assign to the new system.

        Returns:
            An API response payload.
        """
        return await self._client.async_request_and_validate(
            "post", "systems", SystemGetResponse, json={"systems": attributes}
        )

    async def async_delete(self, system_id: int) -> None:
        """Delete a system by ID.

        Args:
            system_id: The ID of the system to delete.
        """
        await self._client.async_request("delete", f"systems/{system_id}")

    async def async_get(self, system_id: int) -> SystemGetResponse:
        """Get a system by ID.

        Args:
            system_id: The ID of the system to get.

        Returns:
            An API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", f"systems/{system_id}", SystemGetResponse
        )

    async def async_update(
        self, system_id: int, new_attributes: dict[str, Any]
    ) -> SystemGetResponse:
        """Update a system with a specific attribute payload.

        Args:
            system_id: The ID of the system to update.
            new_attributes: The new attributes to give the system.

        Returns:
            An API response payload.
        """
        return await self._client.async_request_and_validate(
            "put",
            f"systems/{system_id}",
            SystemGetResponse,
            json={"systems": new_attributes},
        )



"""Define model helpers."""
from typing import TypeVar

from pydantic import BaseModel, ConfigDict


class NotionBaseModel(BaseModel):
    """Define a Notion-specific base model."""

    model_config = ConfigDict(frozen=True)


NotionBaseModelT = TypeVar("NotionBaseModelT", bound=NotionBaseModel)



"""Define endpoints for interacting with users."""
from __future__ import annotations

from typing import TYPE_CHECKING

from aionotion.user.models import UserPreferencesResponse

if TYPE_CHECKING:
    from aionotion.client import Client


class User:  # pylint: disable=too-few-public-methods
    """Define an object to interact with user endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_preferences(self) -> UserPreferencesResponse:
        """Get user preferences.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get",
            f"users/{self._client.user_uuid}/user_preferences",
            UserPreferencesResponse,
        )



"""Define endpoints for interacting with sensors."""
from __future__ import annotations

from typing import TYPE_CHECKING, Any

from aionotion.sensor.models import (
    ListenerAllResponse,
    SensorAllResponse,
    SensorGetResponse,
)

if TYPE_CHECKING:
    from aionotion.client import Client


class Sensor:
    """Define an object to interact with sensor endpoints."""

    def __init__(self, client: Client) -> None:
        """Initialize.

        Args:
            client: The aionotion client
        """
        self._client = client

    async def async_all(self) -> SensorAllResponse:
        """Get all sensors.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", "sensors", SensorAllResponse
        )

    async def async_create(self, attributes: dict[str, Any]) -> SensorGetResponse:
        """Create a sensor with a specific attribute payload.

        Args:
            attributes: The attributes to assign to the new sensor.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "post", "sensors", SensorGetResponse, json={"sensors": attributes}
        )

    async def async_delete(self, sensor_id: int) -> None:
        """Delete a sensor by ID.

        Args:
            sensor_id: The ID of the sensor to delete.
        """
        await self._client.async_request("delete", f"sensors/{sensor_id}")

    async def async_get(self, sensor_id: int) -> SensorGetResponse:
        """Get a sensor by ID.

        Args:
            sensor_id: The ID of the sensor to get.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", f"sensors/{sensor_id}", SensorGetResponse
        )

    async def async_listeners(self) -> ListenerAllResponse:
        """Get all listeners for all sensors.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", "sensor/listeners", ListenerAllResponse
        )

    async def async_listeners_for_sensor(self, sensor_uuid: str) -> ListenerAllResponse:
        """Get all listeners for a sensor by UUID.

        Note that unlike other sensor endpoints, the sensor ID won't work here; the
        sensor UUID is required.

        Args:
            sensor_uuid: The UUID of the sensor to get.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "get", f"sensors/{sensor_uuid}/listeners", ListenerAllResponse
        )

    async def async_update(
        self, sensor_id: int, new_attributes: dict[str, Any]
    ) -> SensorGetResponse:
        """Update a sensor with a specific attribute payload.

        Args:
            sensor_id: The ID of the sensor to update.
            new_attributes: The new attributes to give the sensor.

        Returns:
            A validated API response payload.
        """
        return await self._client.async_request_and_validate(
            "put",
            f"sensors/{sensor_id}",
            SensorGetResponse,
            json={"sensors": new_attributes},
        )
